# -----------------------------------------------------------------------------
# Copyright (c) Peter Bingham  (petersbingham@hotmail.co.uk)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------------

import sys
from . import dae_RelayBoard_Common


class DAE_RelayBoard_D2XX:
    BAUDRATE = 921600
    MASK = 255
    BITMODE = 4

    def __init__(self):
        if sys.platform == "win32":
            from . import FTD2XXWindows

            self.FTD2XX = FTD2XXWindows.FTD2XXWindows()
        elif "linux" in sys.platform:
            from . import FTD2XXLinux

            self.FTD2XX = FTD2XXLinux.FTD2XXLinux()

    # currently only gets first board
    def initialise(self, *args):
        self.disconnect()
        self._checkInit()
        deviceId = dae_RelayBoard_Common.DENKOVI_ID
        if len(args):
            deviceId = args[0]
        self.FTD2XX.initialise(deviceId, self.BAUDRATE, self.MASK, self.BITMODE)

    def disconnect(self):
        self._checkInit()
        self.FTD2XX.close()

    ###############
    ### Setters ###
    ###############

    def setAllStatesOn(self):
        self._setAllStates(True)

    def setAllStatesOff(self):
        self._setAllStates(False)

    def setStates(self, states):
        for relay in states:
            self.setState(relay, states[relay])

    def setState(self, relay, on):
        self._checkInit()
        if relay > self.NUMRELAYS or relay < 1:
            raise dae_RelayBoard_Common.Denkovi_Exception("Relay Index out of bounds")
        stateByte = self._readByte()
        toggle = 1 << self._getBitIndex(relay)
        if on:
            relState = stateByte | toggle
        else:
            relState = stateByte & ~toggle
        self._writeByte(relState)

    def _setAllStates(self, on):
        self._checkInit()
        if on:
            writeChar = 0xFF
        else:
            writeChar = 0
        self._writeByte(writeChar)

    def _writeByte(self, byte):
        return self.FTD2XX.writeByte(byte)

    ###############
    ### Getters ###
    ###############

    def getStates(self):
        self._checkInit()
        stateByte = self._readByte()
        states = {}
        for relay in range(1, self.NUMRELAYS + 1):
            states[relay] = self._getState(stateByte, relay)
        return states

    def getState(self, relay):
        self._checkInit()
        if relay > self.NUMRELAYS or relay < 1:
            raise dae_RelayBoard_Common.Denkovi_Exception("Relay Index out of bounds")
        stateByte = self._readByte()
        return self._getState(stateByte, relay)

    def _getState(self, stateByte, relay):
        if 1 << self._getBitIndex(relay) & stateByte:
            return True
        else:
            return False

    def _readByte(self):
        return self.FTD2XX.readByte()

    ###############
    ### Helpers ###
    ###############

    def _checkInit(self):
        if self.FTD2XX is None:
            raise dae_RelayBoard_Common.Denkovi_Exception(
                "FTD2XX module not initialised"
            )


class DAE_RelayBoard_8(DAE_RelayBoard_D2XX):
    NUMRELAYS = 8

    def _getBitIndex(self, relay):
        return relay - 1


class DAE_RelayBoard_4(DAE_RelayBoard_D2XX):
    NUMRELAYS = 4

    def _getBitIndex(self, relay):
        return 2 * relay - 1



# -----------------------------------------------------------------------------
# Copyright (c) Peter Bingham  (petersbingham@hotmail.co.uk)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------------

# Test and Example Code. Not automated. Monitor relays and check output.

import time
import dae_RelayBoard


def testStateFunctions(dr, relay, state):
    dr.setState(relay, state)
    time.sleep(0.5)
    testGetFunctions(dr)


def testGetFunctions(dr, printNums=True):
    states = dr.getStates()
    allStates = {}
    singleStates = {}
    for relay in states:
        if dr.getState(relay):
            allStates[relay] = "T"
        else:
            allStates[relay] = "F"
        if states[relay]:
            singleStates[relay] = "T"
        else:
            singleStates[relay] = "F"
    if printNums:
        print("All States:\t" + str(allStates.keys()).replace("'", ""))
        print("\t\t" + str(allStates.values()).replace("'", ""))
        print("Single States:\t" + str(singleStates.keys()).replace("'", ""))
        print("\t\t" + str(singleStates.values()).replace("'", ""))
    else:
        print("All States:\t" + str(allStates.values()).replace("'", ""))
        print("Single States:\t" + str(singleStates.values()).replace("'", ""))


class TestCommon:
    def __init__(self, boardType, *args):

        dr = dae_RelayBoard.DAE_RelayBoard(boardType)
        dr.initialise(*args)

        dr.setAllStatesOff()

        print("Turn on one-by-one")
        for relay in range(1, dr.getNumRelays() + 1):
            testStateFunctions(dr, relay, True)

        print("\n\nTurn off one-by-one")
        for relay in range(1, dr.getNumRelays() + 1):
            testStateFunctions(dr, relay, False)

        print("\n\nTurn all on")
        dr.setAllStatesOn()
        testGetFunctions(dr)
        time.sleep(0.5)

        print("\n\nTurn all off")
        dr.setAllStatesOff()
        time.sleep(0.5)
        testGetFunctions(dr)

        print("\n\nTurn even on")
        evenStates = {}
        for relay in range(1, dr.getNumRelays() + 1):
            evenStates[relay] = relay % 2 == 0
        dr.setStates(evenStates)
        time.sleep(0.5)
        testGetFunctions(dr)

        print("\n\nTurn first on")
        dr.setState(1, True)
        time.sleep(0.5)
        testGetFunctions(dr)

        print("\n\nTurn first off")
        dr.setState(1, False)
        time.sleep(0.5)
        testGetFunctions(dr)

        dr.disconnect()



# -----------------------------------------------------------------------------
# Copyright (c) Peter Bingham  (petersbingham@hotmail.co.uk)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------------

from ctypes import *

FT_OK = 0
FT_INVALID_HANDLE = 1
FT_DEVICE_NOT_FOUND = 2
FT_DEVICE_NOT_OPENED = 3
FT_IO_ERROR = 4
FT_INSUFFICIENT_RESOURCES = 5
FT_INVALID_PARAMETER = 6
FT_INVALID_BAUD_RATE = 7
FT_DEVICE_NOT_OPENED_FOR_ERASE = 8
FT_DEVICE_NOT_OPENED_FOR_WRITE = 9
FT_FAILED_TO_WRITE_DEVICE = 10
FT_EEPROM_READ_FAILED = 11
FT_EEPROM_WRITE_FAILED = 12
FT_EEPROM_ERASE_FAILED = 13
FT_EEPROM_NOT_PRESENT = 14
FT_EEPROM_NOT_PROGRAMMED = 15
FT_INVALID_ARGS = 16
FT_NOT_SUPPORTED = 17
FT_OTHER_ERROR = 18


def FTDllErrMap(errCode):
    return {
        FT_OK: "FT_OK",
        FT_INVALID_HANDLE: "FT_INVALID_HANDLE",
        FT_DEVICE_NOT_FOUND: "FT_DEVICE_NOT_FOUND",
        FT_DEVICE_NOT_OPENED: "FT_DEVICE_NOT_OPENED",
        FT_IO_ERROR: "FT_IO_ERROR",
        FT_INSUFFICIENT_RESOURCES: "FT_INSUFFICIENT_RESOURCES",
        FT_INVALID_PARAMETER: "FT_INVALID_PARAMETER",
        FT_INVALID_BAUD_RATE: "FT_INVALID_BAUD_RATE",
        FT_DEVICE_NOT_OPENED_FOR_ERASE: "FT_DEVICE_NOT_OPENED_FOR_ERASE",
        FT_DEVICE_NOT_OPENED_FOR_WRITE: "FT_DEVICE_NOT_OPENED_FOR_WRITE",
        FT_FAILED_TO_WRITE_DEVICE: "FT_FAILED_TO_WRITE_DEVICE",
        FT_EEPROM_READ_FAILED: "FT_EEPROM_READ_FAILED",
        FT_EEPROM_WRITE_FAILED: "FT_EEPROM_WRITE_FAILED",
        FT_EEPROM_ERASE_FAILED: "FT_EEPROM_ERASE_FAILED",
        FT_EEPROM_NOT_PRESENT: "FT_EEPROM_NOT_PRESENT",
        FT_EEPROM_NOT_PROGRAMMED: "FT_EEPROM_NOT_PROGRAMMED",
        FT_INVALID_ARGS: "FT_INVALID_ARGS",
        FT_NOT_SUPPORTED: "FT_NOT_SUPPORTED",
        FT_OTHER_ERROR: "FT_OTHER_ERROR",
    }.get(errCode, "Unknown Error Code: " + str(errCode))


FT_LIST_NUMBER_ONLY = 0x80000000
FT_LIST_BY_INDEX = 0x40000000
FT_LIST_ALL = 0x20000000

FT_OPEN_BY_SERIAL_NUMBER = 1
FT_OPEN_BY_DESCRIPTION = 2
FT_OPEN_BY_LOCATION = 4

MAX_BUFF_SZ = 64


class FTD2XXDllWrap(object):
    def __init__(self):
        self.FTD2XXDLL = WinDLL("FTD2XX.dll")

    def FT_Open(self, deviceNum):
        handle = c_void_p()
        ret = self.FTD2XXDLL.FT_Open(deviceNum, pointer(handle))
        return (ret, handle)

    def FT_Close(self, handle):
        ret = self.FTD2XXDLL.FT_Close(handle)
        return (ret,)

    def FT_Write(self, handle, buff, numBytes):
        buff = pointer(c_byte(buff))
        bytesWritten = c_int()
        ret = self.FTD2XXDLL.FT_Write(
            handle, cast(buff, c_void_p), numBytes, pointer(bytesWritten)
        )
        return (ret, bytesWritten)

    def FT_Read(self, handle, numBytes):
        buff = (c_char * numBytes)()
        ret = self.FTD2XXDLL.FT_Read(handle, cast(buff, c_void_p), numBytes)
        string = ""
        for i in range(0, numBytes):
            string += buff[i]
        return (ret, string)

    def FT_GetNumDevices(self):
        numDevs = c_int()
        ret = self.FTD2XXDLL.FT_CreateDeviceInfoList(byref(numDevs))
        return (ret, numDevs.value)

    def FT_GetDeviceDescription(self, index):
        return self._getBuffer(index, FT_OPEN_BY_DESCRIPTION)

    def FT_GetDeviceSerialNumber(self, index):
        return self._getBuffer(index, FT_OPEN_BY_SERIAL_NUMBER)

    def _getBuffer(self, index, flag):
        index = c_void_p(index)
        buff = (c_char * MAX_BUFF_SZ)()
        ret = self.FTD2XXDLL.FT_ListDevices(
            index, cast(buff, c_void_p), FT_LIST_BY_INDEX | flag
        )
        string = ""
        for i in range(0, MAX_BUFF_SZ):
            if buff[i] != "\x00":
                string += buff[i].decode()
        return (ret, string)

    def FT_SetBitMode(self, handle, mask, enable):
        ret = self.FTD2XXDLL.FT_SetBitMode(handle, mask, enable)
        return (ret,)

    def FT_SetBaudRate(self, handle, baudRate):
        ret = self.FTD2XXDLL.FT_SetBaudRate(handle, baudRate)
        return (ret,)

    def FT_GetBitMode(self, handle):
        charBuff = c_ubyte()
        ret = self.FTD2XXDLL.FT_GetBitMode(handle, pointer(charBuff))
        return (ret, charBuff.value)

    def FT_ResetDevice(self, handle):
        ret = self.FTD2XXDLL.FT_ResetDevice(handle)
        return (ret,)

    ###############
    ### Helpers ###
    ###############

    def initFirst(self, idString):
        ret, numDevs = self.FT_GetNumDevices()
        if ret == FT_OK:
            for i in range(0, numDevs):
                ret, SN = self.FT_GetDeviceSerialNumber(i)
                if ret == FT_OK:
                    if idString.upper() in SN.upper():
                        return self.FT_Open(i)
            return (ret, None)
        else:
            return (ret, None)

    def initSpecified(self, idString):
        indexed = idString.split("#", 2)
        if len(indexed) == 2:
            idString = indexed[0]
            which = int(indexed[1])
        else:
            which = 0

        index = 0
        ret, numDevs = self.FT_GetNumDevices()
        if ret == FT_OK:
            for i in range(0, numDevs):
                ret, SN = self.FT_GetDeviceSerialNumber(i)
                if ret == FT_OK:
                    if idString.upper() in SN.upper():
                        if which == index:
                            return self.FT_Open(i)
                        index += 1
            return (ret, None)
        else:
            return (ret, None)



# -----------------------------------------------------------------------------
# Copyright (c) Peter Bingham  (petersbingham@hotmail.co.uk)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------------

from . import dae_RelayBoard_Common

DAE_RELAYBOARD_TYPE_4 = "type4"
DAE_RELAYBOARD_TYPE_8 = "type8"
DAE_RELAYBOARD_TYPE_16 = "type16"


class DAE_RelayBoard:
    def __init__(self, boardType, *args):
        self.relayHandler = None
        self.CommString = False
        if boardType == DAE_RELAYBOARD_TYPE_16:
            from . import dae_RelayBoard_VCP

            self.CommString = True
            self.relayHandler = dae_RelayBoard_VCP.DAE_RelayBoard_16(*args)
        elif boardType == DAE_RELAYBOARD_TYPE_8:
            from . import dae_RelayBoard_D2XX

            self.relayHandler = dae_RelayBoard_D2XX.DAE_RelayBoard_8(*args)
        elif boardType == DAE_RELAYBOARD_TYPE_4:
            from . import dae_RelayBoard_D2XX

            self.relayHandler = dae_RelayBoard_D2XX.DAE_RelayBoard_4(*args)
        self._checkInit()

    def needsCommString(self):
        return self.CommString

    def initialise(self, *args):
        self._checkInit()
        self.relayHandler.initialise(*args)

    def disconnect(self):
        self._checkInit()
        self.relayHandler.disconnect()

    def getNumRelays(self):
        self._checkInit()
        return self.relayHandler.NUMRELAYS

    ###############
    ### Setters ###
    ###############

    def setAllStatesOn(self):
        self._checkInit()
        self.relayHandler.setAllStatesOn()

    def setAllStatesOff(self):
        self._checkInit()
        self.relayHandler.setAllStatesOff()

    def setStates(self, states):
        self.relayHandler.setStates(states)

    def setState(self, relay, on):
        self._checkInit()
        self.relayHandler.setState(relay, on)

    ###############
    ### Getters ###
    ###############

    def getStates(self):
        self._checkInit()
        return self.relayHandler.getStates()

    def getState(self, relay):
        self._checkInit()
        return self.relayHandler.getState(relay)

    ###############
    ### Helpers ###
    ###############

    def _checkInit(self):
        if self.relayHandler is None:
            raise dae_RelayBoard_Common.Denkovi_Exception(
                "No supported board has been successfully initialised"
            )



# -----------------------------------------------------------------------------
# Copyright (c) Peter Bingham  (petersbingham@hotmail.co.uk)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------------

# Some quick tests. You need to be connected to the board for testing. Unhandled exceptions are a fail. Comment depending on attached board.

# Example Code. Set Com port below:
COMPORT = "COM8"

import time
import dae_RelayBoard
from dae_RelayBoard.dae_RelayBoard_Common import Denkovi_Exception


def testException(function, *args):
    try:
        function(*args)
    except Denkovi_Exception as inst:
        print("Expected Exception: " + str(inst))


def testBoard(boardType, numRelays, devID=None):
    dr = dae_RelayBoard.DAE_RelayBoard(boardType)
    testException(dr.setAllStatesOn)
    if devID is not None:
        dr.initialise(devID)
    else:
        dr.initialise()
    testException(dr.setState, numRelays + 1, True)
    testException(dr.getState, numRelays + 1)
    testException(dr.setState, 0, True)
    testException(dr.getState, 0)
    dr.disconnect()


testException(dae_RelayBoard.DAE_RelayBoard, 99999)

# testBoard(dae_RelayBoard.DAE_RELAYBOARD_TYPE_4, 4)
# testBoard(dae_RelayBoard.DAE_RELAYBOARD_TYPE_8, 8)
testBoard(dae_RelayBoard.DAE_RELAYBOARD_TYPE_16, 16, COMPORT)



# -----------------------------------------------------------------------------
# Copyright (c) Peter Bingham  (petersbingham@hotmail.co.uk)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------------

from . import FTD2XXDllWrap
from . import dae_RelayBoard_Common


class FTD2XXWindows(object):
    def __init__(self):
        self.handle = None
        self.FTD2XXDLL = FTD2XXDllWrap.FTD2XXDllWrap()

    def initialise(self, deviceID, baudRate, mask, bitMode):
        self.handle = self._handleRet(self.FTD2XXDLL.initSpecified(deviceID))[1]
        self._checkInit()
        self._handleRet(self.FTD2XXDLL.FT_SetBaudRate(self.handle, baudRate))
        self._handleRet(self.FTD2XXDLL.FT_SetBitMode(self.handle, mask, bitMode))

    def close(self):
        if self.handle is not None:
            self._handleRet(self.FTD2XXDLL.FT_Close(self.handle))
            self.handle = None

    def writeByte(self, byte):
        self._checkInit()
        self._handleRet(self.FTD2XXDLL.FT_Write(self.handle, byte, 1))

    def readByte(self):
        self._checkInit()
        return self._handleRet(self.FTD2XXDLL.FT_GetBitMode(self.handle))[1]

    def _checkInit(self):
        if self.handle is None:
            raise dae_RelayBoard_Common.Denkovi_Exception(
                "FTD2XXWindows device not initialised"
            )

    def _handleRet(self, ret):
        if ret[0] != FTD2XXDllWrap.FT_OK:
            raise dae_RelayBoard_Common.Denkovi_Exception(self._getErrString(ret))
        return ret

    def _getErrString(self, ret):
        return "FT return error: " + FTD2XXDllWrap.FTDllErrMap(ret[0])



#!/usr/bin/env python

from distutils.core import setup

# Read the contents of your README file
with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="dae_RelayBoard",
    version="1.5.2",
    description="Denkovi Relay Board Controller",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Peter Bingham",
    author_email="petersbingham@hotmail.co.uk",
    url="https://code.google.com/p/dae-py-relay-controller/",
    packages=["dae_RelayBoard"],
)



# -----------------------------------------------------------------------------
# Copyright (c) Peter Bingham  (petersbingham@hotmail.co.uk)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------------

import serial
import time
from . import dae_RelayBoard_Common


class DAE_RelayBoard_VCP:
    TIMEOUT = 1.0
    DELIM = "//"
    DEFAULT_CMDDELAY = 0.05  # Can override this default with zero and control delay in client code. Says 5ms but was getting corruption even around 20ms, so increased to 50ms. See test16Intervals.

    def __init__(self, *args):
        if len(args) > 0:
            self.cmdDelay = args[0]
        else:
            self.cmdDelay = self.DEFAULT_CMDDELAY
        self.ser = None

    def initialise(self, *args):
        self.disconnect()
        self.portstring = args[0]
        self.ser = self._connectToSerial()

    def disconnect(self):
        if self.ser is not None:
            self.ser.close()
            self.ser = None

    def _connectToSerial(self):
        try:
            return serial.Serial(
                port=self.portstring,
                baudrate=9600,
                parity=serial.PARITY_NONE,
                stopbits=serial.STOPBITS_ONE,
                bytesize=serial.EIGHTBITS,
                xonxoff=0,  # no software flow control
                rtscts=0,  # no RTS/CTS flow control
                timeout=self.TIMEOUT,
                writeTimeout=self.TIMEOUT,
            )
        except serial.SerialException:
            raise dae_RelayBoard_Common.Denkovi_Exception(
                "Could not connect to relay board"
            )

    ###############
    ### Setters ###
    ###############

    def setAllStatesOn(self):
        self._setAllStates(True)

    def setAllStatesOff(self):
        self._setAllStates(False)

    def setStates(self, states):
        for relay in states:
            self.setState(relay, states[relay])

    def setState(self, relay, on):
        if relay > self.NUMRELAYS or relay < 1:
            raise dae_RelayBoard_Common.Denkovi_Exception("Relay Index out of bound")
        else:
            self._prepForCmd()
            if on:
                stateString = "+"
            else:
                stateString = "-"
            numString = "%02d" % relay
            self._write(numString + stateString)

    def _setAllStates(self, on):
        self._prepForCmd()
        if on:
            stateString = "on"
        else:
            stateString = "off"
        self._write(stateString)

    ###############
    ### Getters ###
    ###############

    def getStates(self):
        self._prepForCmd()
        stateString = self._getStateString()
        states = {}
        for relay in range(1, self.NUMRELAYS + 1):
            states[relay] = self._getState(relay, stateString)
        return states

    def getState(self, relay):
        if relay > self.NUMRELAYS or relay < 1:
            raise dae_RelayBoard_Common.Denkovi_Exception("Relay Index out of bound")
        else:
            self._prepForCmd()
            stateString = self._getStateString()
            return self._getState(relay, stateString)

    def _getStateString(self):
        self._write("ask")
        time.sleep(self.cmdDelay)
        return self._read(2)

    def _getState(self, relay, stateString):
        if relay > self.NUMRELAYS / 2:
            stateStringIndex = 1
            shifts = relay - self.NUMRELAYS / 2 - 1
        else:
            stateStringIndex = 0
            shifts = relay - 1

        field = 128 >> int(shifts)
        try:
            return field & ord(stateString[stateStringIndex]) > 0  # Python 2
        except TypeError:
            return field & stateString[stateStringIndex] > 0  # Python 3

    ###############
    ### Helpers ###
    ###############

    def _prepForCmd(self):
        if self.ser == None:
            raise dae_RelayBoard_Common.Denkovi_Exception("Board Not Initialised")
        else:
            time.sleep(self.cmdDelay)
            self.ser.flushInput()
            self.ser.flushOutput()

    def _write(self, cmdString):
        try:
            writeString = cmdString + self.DELIM
            bytesWritten = self.ser.write(writeString.encode())
            if len(writeString) != bytesWritten:
                raise dae_RelayBoard_Common.Denkovi_Exception("Error during write")
        except (serial.SerialException, serial.SerialTimeoutException):
            raise dae_RelayBoard_Common.Denkovi_Exception("Error while writing")

    def _read(self, numBytes):
        try:
            retString = self.ser.read(size=numBytes)
            if len(retString) < numBytes:
                raise dae_RelayBoard_Common.Denkovi_Exception(
                    "Timeout waiting for response"
                )
            return retString
        except (serial.SerialException, serial.SerialTimeoutException):
            raise dae_RelayBoard_Common.Denkovi_Exception("Error while writing")


class DAE_RelayBoard_16(DAE_RelayBoard_VCP):
    NUMRELAYS = 16



# -----------------------------------------------------------------------------
# Copyright (c) Peter Bingham  (petersbingham@hotmail.co.uk)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
# -----------------------------------------------------------------------------

from __future__ import print_function

import re
from . import dae_RelayBoard_Common
from pylibftdi import BitBangDevice, Driver


class FTD2XXLinux(object):
    def __init__(self):
        self.driver = Driver()
        self.bb = None
        self.comport = None

    def initialise(self, deviceID, baudRate, mask, bitMode):
        try:
            indexed = deviceID.split("#", 2)
            if len(indexed) == 2:
                deviceID = indexed[0]
                which = int(indexed[1])
            else:
                which = 0

            index = 0
            found = False
            for dev in self.driver.list_devices():
                if re.search(deviceID + ".+", dev[2].decode()):
                    if which == index:
                        found = True
                        print(
                            "Found device %s#%d => %s"
                            % (deviceID, index, dev[2].decode())
                        )
                        deviceID = dev[2].decode()
                        break
                    index += 1

            if found:
                self.bb = BitBangDevice(deviceID)
                self.bb.direction = mask
                self.bb.open()
        except Exception as e:
            raise dae_RelayBoard_Common.Denkovi_Exception(
                "Could not connect to relay board: %s: %s" % (e.__class__, e)
            )
        if not found:
            raise dae_RelayBoard_Common.Denkovi_Exception(
                "Relay board: %s#%d not found" % (deviceID, index)
            )

    def close(self):
        if self.bb is not None:
            self.bb.close()
            self.bb = None

    def writeByte(self, byte):
        if self.bb is None:
            raise dae_RelayBoard_Common.Denkovi_Exception("Board non initialized")
        self.bb.port = byte

    def readByte(self):
        if self.bb is None:
            raise dae_RelayBoard_Common.Denkovi_Exception("Board non initialized")
        return self.bb.port
